import inspect

from trac.core import *
from trac.config import Option
from trac.wiki.api import IWikiMacroProvider
from trac.web.chrome import ITemplateProvider

from pymills.db import Connection
from pymills.table import Table, Header, Row, Cell

__all__ = ["SqlQueryMacro"]

class SqlQueryMacro(Component):
	"""
	A macro to execute an SQL query against a
	configured database and render the results
	in a table.

	Example:
	{{{
	[[SQL(SELECT foo FROM bar)]]
	}}}
	"""
	
	implements(IWikiMacroProvider, ITemplateProvider)

	uri = Option("sqlquery", "uri", "sqlite://:memory:",
		"""Database URI to connect to and use for SQL Queries""")
	
	# IWikiMacroProvider methods
	def get_macros(self):
		yield "SQL"
		
	def get_macro_description(self, name):
		return inspect.getdoc(self.__class__)

	def expand_macro(self, formatter, name, args):
		if args is None:
			return "No query defined!"

		sql = str(args).strip()

		db = Connection(self.uri)
		rows = db.do(sql)

		if rows:
			headers = []
			fields = [x for x in rows[0].keys() if not x.startswith("__") and not x.endswith("__")]
			for field in fields:
				headers.append(Header(field))
			t = Table(headers, cls="wiki")
			for row in rows:
				style = ""

				if row.has_key("__color__"):
					style += "background-color: %s" % row["__color__"]
					row.remove("__color__")

				r = Row([Cell(v) for v in row.values()],
					style=style)
				t.append(r)
			t.refresh()

			#return "<pre>\n%s\n</pre>" % str(t)
			return t.toHTML()
		else:
			return "No results"

	# ITemplateProvider methods
	def get_templates_dirs(self):
		return []
		
	def get_htdocs_dirs(self):
		from pkg_resources import resource_filename
		return [("sqlquery", resource_filename(__name__, "htdocs"))]
